﻿--[[

Copyright (C) 2004 by neuron
aagaande AT gmail DOT com

This program is free software; you can redistribute it and--or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the
Free Software Foundation, Inc.,
59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

]]

local nQuestLog = nQuestLog

local REVISION = tonumber(("$Revision: 55622 $"):match("%d+"))
if ( nQuestLog.revision < REVISION ) then
	nQuestLog.version = (nQuestLog.versionstring):format(REVISION)
	nQuestLog.revision = REVISION
	nQuestLog.date = ("$Date: 2007-11-21 13:29:16 -0500 (Wed, 21 Nov 2007) $"):match("%d%d%d%d%-%d%d%-%d%d")
end

local moduleName = "Comments"
local Comments = nQuestLog:NewModule(moduleName, "AceEvent-2.0", "AceHook-2.1")

--Load libraries
local Abacus = AceLibrary("Abacus-2.0")
local Dewdrop = AceLibrary("Dewdrop-2.0")
local Quixote = AceLibrary("Quixote-1.0")
local Tablet = AceLibrary("Tablet-2.0")

--Load localization
local L = AceLibrary("AceLocale-2.2"):new("Comments")

function Comments:OnModuleLoad()
	self.menuName = L["Comments"]
	self.description = L["Description"]
	if ( not self.defaultsRegistered ) then
		self.core:RegisterDefaults(moduleName, "profile", { enabled = true,
			data = { fontSizePercent = 1.0, detached = true },
			minWidth = 300, maxHeight = 400 })
		self.defaultsRegistered = true
	end
end

function Comments:ToggleEnabled(v)
	self.db.profile.enabled = v
	if ( v ) then
		self:SecureHook(nQuestLog, "ShowLog")
		if ( nQuestLog:IsModuleActive("DetailsFrame") ) then
			self:SecureHook(nQuestLog:GetModule("DetailsFrame"), "UpdateQuestDetail")
		end
	else
		self:Unhook(nQuestLog, "ShowLog")
		if ( nQuestLog:IsModuleActive("DetailsFrame") ) then
			self:Unhook(nQuestLog:GetModule("DetailsFrame"), "UpdateQuestDetail")
		end
	end
end

function Comments:GetMenu(oldMenu)
	local menu = { order = oldMenu.order, type = "group",
		name = self.menuName,
		desc = self.description,
		cmdHidden = true,
		args = {
			toggleEnabled = oldMenu,
			minWidth = { order = oldMenu.order + 1, type = "range",
				name = L["Minimum width"],
				desc = L["Set the minimum width of the Comments frame"],
				min = 50, max = 500, step = 1,
				get = function() return self.db.profile.minWidth end,
				set = function(v)
					self.db.profile.minWidth = v
					if ( Tablet:IsRegistered("nQuestLog_Comments") ) then
						Tablet:Close("nQuestLog_Comments")
						Tablet:Unregister("nQuestLog_Comments")
					end
					self:Update()
				end },
			maxHeight = { order = oldMenu.order + 2, type = "range",
				name = L["Maximum height"],
				desc = L["Set the maximum height of the Comments frame"],
				min = 50, max = 500, step = 1,
				get = function() return self.db.profile.maxHeight end,
				set = function(v)
					self.db.profile.maxHeight = v
					if ( Tablet:IsRegistered("nQuestLog_Comments") ) then
						Tablet:Close("nQuestLog_Comments")
						Tablet:Unregister("nQuestLog_Comments")
					end
					self:Update()
				end
			} } }
	return menu
end

function Comments:ShowLog(object, questid)
	if ( LightHeaded ) and ( nQuestLog:IsModuleEnabled("DetailsFrame") ) and ( questid ) then
		self:ScheduleEvent(self.Update, 0, self) --Schedule an event because tablet bugs out otherwise.
		self.altTitle, self.altLevel = "", nil
	end
end

function Comments:UpdateQuestDetail(object)
	if ( LightHeaded ) then
		self:Update()
	end
end

function Comments:GetQuestId()
	local questid = 1
	if ( nQuestLog:IsModuleEnabled("DetailsFrame") ) then
		questid = nQuestLog:GetModule("DetailsFrame").questid
	end
	return questid
end

function Comments:Update()
	if ( Tablet:IsRegistered("nQuestLog_Comments") ) then
		--Disable the lock every time the tablet is shown
		--I'd rather remove the "Lock" option from the Tablet Dewdrop menu, but that doesn't appear to be possible at this time without modification of Tablet-2.0
		if ( Tablet:IsLocked("nQuestLog_Comments") ) then
			Tablet:ToggleLocked("nQuestLog_Comments")
		end
		--Refresh the Comments pane
		Tablet:Refresh("nQuestLog_Comments")
		return
	end
	
	self.altID, self.altTitle, self.altLevel = nil, "", nil
	
	local badQIDs = {}
	local indentSize = 14
	local DATA_COLOR = "|cFF66FF66"
	local LABEL_COLOR = nQuestLog:RGBPercToHex(NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b)
	local WHITE = "|cFFFFFFFF"
	
	--Add function pointers to core functions used in this module
	local function addWayPoint(...) nQuestLog:AddWayPoint(...) end
	local function itemClick(...) nQuestLog:ItemClick(...) end
	local function itemHover(...) nQuestLog:ItemHover(...) end
	local function RGBPercToHex(...) return nQuestLog:RGBPercToHex(...) end
	local function round(...) return nQuestLog:Round(...) end
	local function sendToChat(...) nQuestLog:SendToChat(...) end
	
	local function buildItemString(itemid)
		return "item:"..itemid..":0:0:0:0:0:0:0"
	end
	
	local function toggleDisplay(id, idtype)
		if ( idtype == "npc" ) then
			if ( self.npcid == id ) then
				self.npcid = nil
			else
				self.npcid = id
			end
		elseif ( idtype == "quest" ) then
			if ( self.qid == id ) then
				self.qid = nil
			else
				self.qid = id
			end
		elseif ( idtype == "object" ) then
			if ( self.objid == id ) then
				self.objid = nil
			else
				self.objid = id
			end
		end
		self:Update()
	end
	
	local function showAltQuest(id)
		LightHeaded:LoadQIDData(id)
		if ( LH_QuestData ) then
			local name, lvl = LightHeaded:QIDToTitleLevel(id)
			self.altID = id
			self.altTitle = name
			self.altLevel = tonumber(lvl)
			self:Update()
		end
	end
	
	local function addURLs(cat, linkid, linktype)
		local aURLtitle, aURL = "-> "..DATA_COLOR.."Allakhazam URL|r <-", "http://wow.allakhazam.com/"
		local tURLtitle, tURL = "-> "..DATA_COLOR.."Thottbot URL|r <-", "http://www.thottbot.com/"
		local wURLtitle, wURL = "-> "..DATA_COLOR.."Wowhead URL|r <-", "http://www.wowhead.com/"
		cat:AddLine("text", wURLtitle, "func", sendToChat, "arg1", wURL.."?"..linktype.."="..linkid, "arg2", true, "indentation", indentSize * 2)
		if ( linktype == "npc" ) then
			cat:AddLine("text", tURLtitle, "func", sendToChat, "arg1", tURL.."c"..linkid, "arg2", true, "indentation", indentSize * 2)
			cat:AddLine("text", aURLtitle, "func", sendToChat, "arg1", aURL.."db/mob.html?wmob="..linkid, "arg2", true, "indentation", indentSize * 2)
		elseif ( linktype == "object" ) then
			cat:AddLine("text", tURLtitle, "func", sendToChat, "arg1", tURL.."o"..linkid, "arg2", true, "indentation", indentSize * 2)
		elseif ( linktype == "quest" ) then
			cat:AddLine("text", tURLtitle, "func", sendToChat, "arg1", tURL.."q"..linkid, "arg2", true, "indentation", indentSize * 2)
			cat:AddLine("text", aURLtitle, "func", sendToChat, "arg1", aURL.."db/quest.html?wquest="..linkid, "arg2", true, "indentation", indentSize * 2)
		end
	end
	
	local function addNpcLocs(cat, id, name, qTitle, qLevel)
		id = tonumber(id)
		if ( LightHeaded:LoadNPCData(id) ) then
			for c, z, x, y in LightHeaded:IterateNPCLocs(id) do
				local zoneNames = { GetMapZones(c) }
				local zone = zoneNames[z]
				cat:AddLine("text", DATA_COLOR..zone.." ["..x..", "..y.."]|r", "indentation", indentSize * 2, "tooltipTitle", "",
					"func", addWayPoint, "arg1", tonumber(x), "arg2", tonumber(y), "arg3", qTitle,
					"arg4", qLevel, "arg5", name, "arg6", tonumber(c), "arg7", tonumber(z), "arg8", zone)
			end
			return true
		end
		return false
	end
	
	local function popupLinks(links, qTitle, qLevel)
		Dewdrop:Open("Comments", "cursorX", true, "cursorY", true, "children",
			function()
				for i=1, #(links), 1 do
					local x, y, url, name = links[i].x, links[i].y, links[i].url, links[i].name
					if ( x ) and ( y ) then
						Dewdrop:AddLine("text", LABEL_COLOR.."Coordinates:  "..DATA_COLOR.."["..x..", "..y.."]", "notCheckable", true, "tooltipTitle", "",
							"func", addWayPoint, "arg1", tonumber(x), "arg2", tonumber(y), "arg3", qTitle, "arg4", qLevel)
					elseif ( url ) then
						Dewdrop:AddLine("text", LABEL_COLOR.."URL:  "..DATA_COLOR..url, "notCheckable", true, "tooltipTitle", "",
							"func", sendToChat, "arg1", url, "arg2", true)
						local _, _, urlType, urlID = url:find("http://www.wowhead.com/%?(%a+)=(%d+)")
						if ( urlID ) then
							urlID = tonumber(urlID)
							if ( urlType == "quest" ) then
								local questValid = true
								for _, v in ipairs(badQIDs) do
									if ( v == urlID ) then
										questValid = false
										break
									end
								end
								if ( questValid ) then
									local link
									if ( LightHeaded:LoadQIDData(urlID) ) then
										local name, lvl = LightHeaded:QIDToTitleLevel(urlID)
										if ( name ) and ( lvl ) then
											local color = GetDifficultyColor(lvl)
											link = RGBPercToHex(color.r, color.g, color.b).."["..lvl.."] "..name
										end
									end
									if ( not link ) then
										link = LABEL_COLOR.."Quest "..urlID
									end
									Dewdrop:AddLine("text", link, "func", showAltQuest, "arg1", urlID, "tooltipTitle", "")
								end
							end
							if ( urlType == "item" ) then
								local itemString = buildItemString(urlID)
								local name, link, rarity = GetItemInfo(itemString)
								local unsafe = false
								if ( not name ) then
									-- Enable unsafe item link
									link = itemString
									name = L["<not yet cached>"]
									unsafe = true
								end
								local item = "["..name.."]"
								if ( rarity ) then
									local _, _, _, itemColor = GetItemQualityColor(rarity)
									item = itemColor..item
								end
								Dewdrop:AddLine("text", LABEL_COLOR..item, "func", itemClick, "arg1", link,
									"tooltipFunc", itemHover, "tooltipArg1", link, "tooltipArg2", unsafe, "tooltipArg3", urlID)
							end
							if ( urlType == "npc" ) then
								if ( name ) then
									Dewdrop:AddLine("text", WHITE..name, "isTitle", true, "tooltipTitle", "")
								else
									name = "NPC".." "..urlID --TODO: localize
								end
								addNpcLocs(Dewdrop, urlID, name, qTitle, qLevel)
							end
						end
					end
				end;
			end
		)
	end
	
	local function getMobMapQuestIdByName(questname, mmQID)
		if ( mobmap_questnames ) then
			for k, v in pairs(mobmap_questnames) do
				if ( v == questname ) and ( k > tonumber(mmQID) ) then
					return k
				end
			end
		end
		return nil
	end
	
	Tablet:Register("nQuestLog_Comments", "detachedData", self.db.profile.data, "cantAttach", true, "dontHook", true, "hideWhenEmpty", true,
		"showTitleWhenDetached", true, "strata", "HIGH", "minWidth", self.db.profile.minWidth, "maxHeight", self.db.profile.maxHeight, "children",
		function()
			local questid = self:GetQuestId()
			if ( questid ) and ( LightHeaded.GetQuestInfo ) and ( LightHeaded.IterateQuestComments ) and ( LightHeaded.IterateQuestInfo ) then
				local title, level, tag = Quixote:GetQuestById(questid)
				local qid, sharable, _, reqlev, stype, sname, sid, etype, ename, eid, xp, rep, series
				if ( self.altID ~= nil ) and ( self.altTitle ~= "" ) and ( self.altLevel ~= nil ) then
					for iqid, isharable, _, ireqlev, istype, isname, isid, ietype, iename, ieid, ixp, irep, iseries in
						LightHeaded:IterateQuestInfo(self.altTitle, self.altLevel) do
						if ( tonumber(iqid) == tonumber(self.altID) ) then
							qid, sharable, reqlev, stype, sname, sid, etype, ename, eid, xp, rep, series =
								iqid, isharable, ireqlev, istype, isname, isid, ietype, iename, ieid, ixp, irep, iseries
							break
						end
					end
					if ( qid ) then
						title = self.altTitle
						level = self.altLevel
					else
						tinsert(badQIDs, self.altID)
					end
				end
				if ( not qid ) then
					qid, sharable, _, reqlev, stype, sname, sid, etype, ename, eid, xp, rep, series = LightHeaded:GetQuestInfo(title, level)
					if ( not qid ) then
						return
					end
				end
				level, qid = tonumber(level), tonumber(qid)
				local qColor = GetDifficultyColor(level)
				
				--TITLE  |  Comments  |
				Tablet:SetTitle("Comments") --Need to localize
				
				--CATEGORY  |  LightHeaded Info  |  Level, title, color, id, reqlvl, sharable, start, end
				local cat = Tablet:AddCategory("text", "\nLightHeaded Info", "columns", 1, "wrap", true) --Need to localize
				
				--LINE  |  [<quest level>] <quest title>  |  Colored by difficulty
				cat:AddLine("text", "["..level.."] "..title, "textR", qColor.r or 1, "textG", qColor.g or 1, "textB", qColor.b or 1, "indentation", indentSize)
				
				--LINE  |  Quest ID [<quest id>] - Req Lvl <required level> - Sharable  |  Clickable
				local qInfo = "Quest ID "..DATA_COLOR.."["..qid.."]|r" --Need to localize
				if ( reqlev ~= "" ) then
					qInfo = qInfo.." - Req Lvl "..DATA_COLOR..reqlev.."|r" --Need to localize
				end
				if ( sharable ~= "" ) then
					qInfo = qInfo.." - "..DATA_COLOR.."Sharable" --Need to localize
				end
				cat:AddLine("text", qInfo, "func", toggleDisplay, "arg1", qid, "arg2", "quest", "indentation", indentSize)
				if ( qid == self.qid ) then
					--LINES  |  -> <site> URL <-  |  Add clickable URLs for the quest when the "Quest ID" line is clicked
					addURLs(cat, qid, "quest")
				end
				
				if ( sname ~= "" ) then
					--LINE  |  Starts: [<starting npc/object>]  |  Clickable
					cat:AddLine("text", "Starts: "..DATA_COLOR.."["..sname.."]", "func", toggleDisplay, "arg1", sid, "arg2", stype, "indentation", indentSize) --Need to localize
					if ( sid == self.npcid ) then
						--LINES  |  <zone> [<xcoord>, <ycoord>]  |  Add clickable zone coordinates for the NPC when the "Starts:" line is clicked
						local npcValid = addNpcLocs(cat, sid, sname, title, level)
						if ( not npcValid ) then
							addURLs(cat, sid, "npc")
						end
					end
					if ( sid == self.objid ) then
						--LINES  |  -> <site> URL <-  |  Add clickable URLs for the object when the "Starts:" line is clicked
						addURLs(cat, sid, stype)
					end
				end
				
				if ( ename ~= "" ) then
					--LINE  |  Ends: [<ending npc/object>]  |  Clickable
					cat:AddLine("text", "Ends: "..DATA_COLOR.."["..ename.."]", "func", toggleDisplay, "arg1", eid, "arg2", etype, "indentation", indentSize) --Need to localize
					if ( eid == self.npcid ) then
						--LINES  |  <zone> [<xcoord>, <ycoord>]  |  Add clickable zone coordinates for the NPC when the "Ends:" line is clicked
						local npcValid = addNpcLocs(cat, eid, ename, title, level)
						if ( not npcValid ) then
							addURLs(cat, eid, "npc")
						end
					end
					if ( eid == self.objid ) then
						--LINES  |  -> <site> URL <-  |  Add clickable URLs for the object when the "Ends:" line is clicked
						addURLs(cat, eid, etype)
					end
				end
				
				if ( xp ~= "" ) or ( rep ~= "" ) then
					--CATEGORY  |  LightHeaded Rewards  |  XP and rep rewards
					cat = Tablet:AddCategory("text", "LightHeaded Rewards", "columns", 1) --Need to localize
					
					if ( xp ~= "" ) then
						local effXP
						local playerLevel = UnitLevel("player")
						local xpLine = "XP: "..DATA_COLOR..xp --Need to localize
						--Quest XP formula from http://www.wowwiki.com/Formulas:Quest_XP
						--Could not determine the exact formula, but this one works well enough
						--Estimates are accurate within 3% based on limited testing
						if ( playerLevel == (level + 6) ) then
							effXP = round(xp*0.8 / 5, 0) * 5
						elseif ( playerLevel == (level + 7) ) then
							effXP = round(xp*0.6 / 5, 0) * 5
						elseif ( playerLevel == (level + 8) ) then
							effXP = round(xp*0.4 / 5, 0) * 5
						elseif ( playerLevel == (level + 9) ) then
							effXP = round(xp*0.2 / 5, 0) * 5
						elseif ( playerLevel >= (level + 10) ) then
							effXP = round(xp*0.1 / 5, 0) * 5
						end
						if ( effXP ) then
							xpLine = xpLine.."|r ("..DATA_COLOR..effXP.."|r @ level "..playerLevel..")" --Need to localize
						end
						cat:AddLine("text", xpLine, "indentation", indentSize)
					end
					
					if ( rep ~= "" ) then
						local factionTable = {}
						for factionIndex=1, GetNumFactions() do
							local name, _, standingId = GetFactionInfo(factionIndex)
							tinsert(factionTable, {name, standingId})
						end
						for name, value in rep:gmatch("([^\029]+)\029([^\029]+)") do
							for _, v in ipairs(factionTable) do
								if ( name == v[1] ) then
									local r, g, b = FACTION_BAR_COLORS[v[2]].r, FACTION_BAR_COLORS[v[2]].g, FACTION_BAR_COLORS[v[2]].b
									local hexcolor = ("%02x%02x%02x"):format(r*255, g*255, b*255)
									cat:AddLine("text", "Rep: |cFF"..hexcolor..name..DATA_COLOR.." +"..value, "indentation", indentSize) --Need to localize
								end
							end
						end
					end
				end --if ( xp ~= "" ) or ( rep ~= "" )
				
				if ( MobMapDotParentFrame ) then
					if ( MobMap_LoadDatabase ) then
						if ( not IsAddOnLoaded("MobMapDatabaseStub"..MOBMAP_QUEST_DATABASE) ) then
							MobMap_LoadDatabase(MOBMAP_QUEST_DATABASE)
						end
					end
					local mmQID = getMobMapQuestIdByName(title, 0)
					local mmLevel, mmZone, mmHorde, mmAlliance, mmMoney, mmNPC, mmSource, mmType, mmAlways, mmChoice
					if ( mmQID ) and ( MobMap_GetDetailsForQuest ) then
						if ( sname ~= "" ) and ( MobMap_GetMobName ) and ( MobMap_GetQuestSourceName ) then
							local l, z, h, a, m, n, s, t, aw, c, lastQID
							while not (( tonumber(mmLevel) == level ) and (( sname == MobMap_GetMobName(mmNPC) ) or ( sname == MobMap_GetQuestSourceName(mmSource) ))) do
								l, z, _, _, h, a, m, n, s, t, aw, c = MobMap_GetDetailsForQuest(mmQID)
								if ( tonumber(l) == level ) then
									mmLevel, mmZone, mmHorde, mmAlliance, mmMoney, mmNPC, mmSource, mmType, mmAlways, mmChoice = l, z, h, a, m, n, s, t, aw, c
									lastQID = mmQID
								end
								mmQID = getMobMapQuestIdByName(title, mmQID)
								if ( mmQID == nil ) then
									mmQID = lastQID
									break
								end
							end
						else
							while ( tonumber(mmLevel) ~= level ) do
								mmQID = getMobMapQuestIdByName(title, mmQID)
								if ( mmQID == nil ) then
									break
								end
								mmLevel, mmZone, _, _, mmHorde, mmAlliance, mmMoney, mmNPC, mmSource, mmType, mmAlways, mmChoice = MobMap_GetDetailsForQuest(mmQID)
							end
						end
					end
					if ( mmQID ) then
						cat = Tablet:AddCategory("text", "MobMap Info", "columns", 1) --Need to localize
						local faction = ""
						if ( mmHorde ) and ( mmAlliance ) then
							faction = " |cFFFFFF00Horde/Alliance" --Need to localize
						elseif ( mmHorde ) then
							faction = " |cFF40BF40Horde-only" --Need to localize
						elseif ( mmAlliance ) then
							faction = " |cFF40BF40Alliance-only" --Need to localize
						end
						cat:AddLine("text", "Quest Type: "..DATA_COLOR..MOBMAP_QUEST_TYPES[mmType]..faction, "indentation", indentSize) --Need to localize
						if ( MobMap_GetZoneName ) then
							cat:AddLine("text", "Zone: "..DATA_COLOR..MobMap_GetZoneName(mmZone), "indentation", indentSize) --Need to localize
						end
						if ( sname == "" ) then
							local source = ""
							if ( mmSource ~= 0 ) and ( MobMap_GetQuestSourceName ) then
								source = MobMap_GetQuestSourceName(mmSource)
							elseif ( mmNPC ~= 0 ) and ( MobMap_GetMobName ) then
								source = MobMap_GetMobName(mmNPC)
							end
							if ( source ~= "" ) and ( MobMap_ParseQuestObjective ) then
								cat:AddLine("text", "Source: "..DATA_COLOR.."["..source.."]", "func", --Need to localize
									function() MobMap_ParseQuestObjective(source) end, "indentation", indentSize)
							end
						end
						if ( xp ~= "" ) or ( #(mmChoice) > 0 ) or ( #(mmAlways) > 0 ) or ( mmMoney > 0 ) then
							local function addRewardItems(rewardTable)
								for _, id in pairs(rewardTable) do
									local itemString = buildItemString(id)
									local name, link, rarity, _, _, _, _, _, _, texture = GetItemInfo(itemString)
									local unsafe = false
									if ( not name ) then
										-- Enable unsafe item link
										link = itemString
										name = L["<not yet cached>"]
										unsafe = true
									end
									local item = "["..name.."]"
									if ( rarity ) then
										local _, _, _, itemColor = GetItemQualityColor(rarity)
										item = itemColor..item
									end
									cat:AddLine("text", item, "checked", true, "hasCheck", true, "checkIcon", texture,
										"indentation", indentSize + 2, "func", itemClick, "arg1", link,
										"onEnterFunc", itemHover, "onEnterArg1", link, "onEnterArg2", unsafe, "onEnterArg3", id,
										"onLeaveFunc", GameTooltip.Hide, "onLeaveArg1", GameTooltip)
								end
							end
							
							cat = Tablet:AddCategory("text", "MobMap Rewards", "columns", 1) --Need to localize
							if ( self.altID ) then
								if ( #(mmChoice) > 0 ) then
									cat:AddLine("text", "Choose from:", "indentation", indentSize) --Need to localize
									addRewardItems(mmChoice)
								end
								if ( #(mmAlways) > 0 ) then
									cat:AddLine("text", "Always receive:", "indentation", indentSize) --Need to localize
									addRewardItems(mmAlways)
								end
							end
							local xpToCopper = 0
							if ( xp ~= "" ) then
								if ( tonumber(xp) >= 1000 ) then
									xpToCopper = round((xp+1)*0.01, 0)*60
								else
									xpToCopper = round((xp+1)*0.02, 0)*30
								end
							end
							if ( mmMoney > 0 ) then
								mmMoney = mmMoney - xpToCopper
							else
								mmMoney = 0
							end
							local money = Abacus:FormatMoneyFull(mmMoney, true)
							if ( xpToCopper > 0 ) then
								money = money.." (+"..Abacus:FormatMoneyFull(xpToCopper, true).." @ 70)" --Need to localize
							end
							cat:AddLine("text", "Money: "..WHITE..money, "indentation", indentSize) --Need to localize
						end --if ( #(mmChoice) > 0 ) or ( #(mmAlways) > 0 ) or ( mmMoney > 0 )
					end --if ( mmQID )
				end --if ( MobMapDotParentFrame )
				
				if ( series ~= "" ) then
					cat = Tablet:AddCategory("text", "LightHeaded Quest Series", "columns", 1) --Need to localize
					
					for step, id in series:gmatch("([^\029]+)\029([^\029]+)") do
						local questValid = true
						id = tonumber(id)
						for _, v in ipairs(badQIDs) do
							if ( v == id ) then
								questValid = false
								break
							end
						end
						if ( questValid ) then
							local link
							local isChecked = false
							if ( LightHeaded:LoadQIDData(id) ) then
								local name, lvl = LightHeaded:QIDToTitleLevel(id)
								if ( name ) and ( lvl ) then
									local color = GetDifficultyColor(lvl)
									link = RGBPercToHex(color.r, color.g, color.b).."["..lvl.."] "..name.."|r"
								end
							end
							if ( not link ) then
								link = step..". ".."Quest "..id --Need to localize
							end
							if ( tonumber(qid) == id ) then
								isChecked = true
							end
							cat:AddLine("text", link, "func", showAltQuest, "arg1", id, "hasCheck", true, "checked", isChecked)
						end
					end
				end --if ( series ~= "" )
				
				cat = Tablet:AddCategory("text", "LightHeaded Comments", "columns", 2) --Need to localize
				if ( LightHeaded:GetNumQuestComments(title, level) ) then
					qid = tonumber(qid)
					for idx, cqid, cid, rating, indent, parent, date, poster, comment in LightHeaded:IterateQuestComments(title, level) do
						if ( qid == tonumber(cqid) ) then
							cat:AddLine("text", (DATA_COLOR.."[%s - %s - %s]|r"):format(poster, date, rating),
								"wrap", true, "indentation", indent*indentSize)
							local links = {}
							for url, name in comment:gmatch("[%[url=]*(http://[^%s%[%]%(%)]+)[[%]]([^%[]+)%[/url%]]*") do
								tinsert(links, { url = url, name = name })
							end
							for xCoord, yCoord in comment:gmatch("(%d%d?%.?%d?%d?)%s*[, ]%s*(%d%d?%.?%d?%d?)") do
								tinsert(links, { x = xCoord, y = yCoord })
							end
							if ( #(links) > 0 ) then
								cat:AddLine("text", comment, "wrap", true, "indentation", indent*indentSize,
									"func", popupLinks, "arg1", links, "arg2", title, "arg3", level)
							else
								cat:AddLine("text", comment, "wrap", true, "indentation", indent*indentSize)
							end
							cat:AddLine("text", " ") --spacer
						end
					end
				else
					cat:AddLine("text", L["No comments found..."])
				end --if ( LightHeaded:GetNumQuestComments(title, level) )
				
				cat:AddLine("text", "Comments are all from Wowhead", "hasCheck", true, "checked", true,
					"checkIcon", "Interface\\Addons\\LightHeaded\\images\\wh_icon", "func", sendToChat, "arg1", "http://www.wowhead.com", "arg2", true)
			end --if ( questid ) and ( LightHeaded.GetQuestInfo ) and ( LightHeaded.IterateQuestComments ) and ( LightHeaded.IterateQuestInfo )
		end) --Tablet:Register()
	Tablet:Open("nQuestLog_Comments")
end