--[[

Copyright (C) 2004 by neuron
aagaande AT gmail DOT com

This program is free software; you can redistribute it and--or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the
Free Software Foundation, Inc.,
59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

]]

local nQuestLog = nQuestLog

local REVISION = tonumber(("$Revision: 55622 $"):match("%d+"))
if ( nQuestLog.revision < REVISION ) then
	nQuestLog.version = (nQuestLog.versionstring):format(REVISION)
	nQuestLog.revision = REVISION
	nQuestLog.date = ("$Date: 2007-11-21 13:29:16 -0500 (Wed, 21 Nov 2007) $"):match("%d%d%d%d%-%d%d%-%d%d")
end

local moduleName = "SmartVisibility"
local SmartVisibility = nQuestLog:NewModule(moduleName, "AceConsole-2.0", "AceEvent-2.0", "AceHook-2.1" )

--Load libraries
local Quixote = AceLibrary("Quixote-1.0")

--Load localization
local L = AceLibrary("AceLocale-2.2"):new("SmartVisibility")

function SmartVisibility:OnModuleLoad()
	self.menuName = L["SmartVisibility"]
	self.description = L["Description"]
	if (not self.defaultsRegistered) then
		self.core:RegisterDefaults(moduleName, "profile", { enabled = true,
			zoneAutoFold = true, zoneAutoUnfold = true,
			hideOldObjectives = 30, --This translates to hiding quest objectives older than 30 minutes, 0 = disable.
			newOnGained = false, --Count gained quests as new quest
			showCurrentZone = false })
		self.defaultsRegistered = true
	end
end

function SmartVisibility:ToggleEnabled(v)
	self.questTimes = {} --stores last update to quest
	self.db.profile.enabled = v
	if ( v ) then
		self:RegisterEvent("Quixote_Quest_Gained", "OnQuest_Gained")
		self:RegisterEvent("Quixote_Quest_Lost", "OnQuest_Lost")
		self:RegisterEvent("Quixote_Leaderboard_Update", "OnQuest_Leaderboard_Update")
		
		self:RegisterEvent("ZONE_CHANGED", "OnZoneChange")
		self:RegisterEvent("ZONE_CHANGED_INDOORS", "OnZoneChange")
		self:RegisterEvent("ZONE_CHANGED_NEW_AREA", "OnZoneChange")
		self:RegisterEvent("Quixote_Ready", "OnZoneChange",true)
	else
		self:UnregisterEvent("Quixote_Quest_Gained", "OnQuest_Gained")
		self:UnregisterEvent("Quixote_Quest_Lost", "OnQuest_Lost")
		self:UnregisterEvent("Quixote_Leaderboard_Update", "OnQuest_Leaderboard_Update")
		
		self:UnregisterEvent("ZONE_CHANGED", "OnZoneChange")
		self:UnregisterEvent("ZONE_CHANGED_INDOORS", "OnZoneChange")
		self:UnregisterEvent("ZONE_CHANGED_NEW_AREA", "OnZoneChange")
	end
	self.core:Refresh()
end

function SmartVisibility:GetMenu(oldMenu)
	local menu = { order = oldMenu.order, type = "group",
		name = self.menuName, desc = self.description,
		cmdHidden = true,
		args = {
			toggleEnabled = oldMenu,
			zoneSpacer = { order = oldMenu.order + 1, type = "header" },
			zoneHeader = { order = oldMenu.order + 2, type = "header", name = L["Zone"] },
			autoFold = { order = oldMenu.order + 3, type = "toggle",
				name = L["Auto fold zones"],
				desc = L["Auto fold zones desc"],
				get = function() return self.db.profile.zoneAutoFold end,
				set = function(v)
					self.db.profile.zoneAutoFold = v
					self:OnZoneChange()
				end },
			autoUnfold = { order = oldMenu.order + 4, type = "toggle",
				name = L["Auto unfold zones"],
				desc = L["Auto fold zones you enter"],
				get = function() return self.db.profile.zoneAutoUnfold end,
				set = function(v)
					self.db.profile.zoneAutoUnfold = v
					self:OnZoneChange()
				end },
			objectivesSpacer = { order = oldMenu.order + 5, type = "header" },
			objectivesHeader = { order = oldMenu.order + 6, type = "header", name = L["Objectives"] },
			currentZone = { order = oldMenu.order + 7, type = "toggle",
				name = L["showCurrentZone"],
				desc = L["showCurrentZoneLong"],
				get = function() return self.db.profile.showCurrentZone end,
				set = function(v)
					self.db.profile.showCurrentZone = v
					self.core:Refresh()
				end },
			hideOld = { order = oldMenu.order + 8, type = "range",
				name = L["Hide old quest objectives"],
				desc = L["Hide quest objectives older than X minutes"].." ("..L["set to 0 to disable"]..")"
					.."\n"..L["Note : This will only work if quests objectives -> hide all objectives is on"],
				min = 0, max = 120, step = 5,
				get = function() return self.db.profile.hideOldObjectives end,
				set = function(v)
					self.db.profile.hideOldObjectives = v
					self.core:Refresh()
				end },
			newOnGained = { order = oldMenu.order + 9, type = "toggle",
				name = " - "..L["Show objectives for new quests"],
				desc = L["Show objectives for new quests"],
				get = function() return self.db.profile.newOnGained end,
				set = function(v)
					self.db.profile.newOnGained = v
					self.core:Refresh()
				end } } }
	return menu
end

function SmartVisibility:OnZoneChange()
	local currentZone = GetZoneText()
	if ( currentZone == "" ) then return end
	--We dont wanna fold stuff if we're changing subzones
	if ( self.currentZone ) and ( self.currentZone == currentZone ) then return end
	self.currentZone = currentZone
	for _, zone in Quixote:IterZones() do
		--Auto fold zone we're not in
		if ( self.db.profile.zoneAutoFold ) and ( zone ~= currentZone ) then
			self.core.db.char.hiddenZones[zone] = true
		--Auto unfold zone we are in
		elseif ( self.db.profile.zoneAutoUnfold ) and ( zone == currentZone ) then
			self.core.db.char.hiddenZones[zone] = nil
		end
	end
	self.core:Refresh() --TODO: find a way for modules to mark needed refresh, and then do one refresh instead of many.
end

function SmartVisibility:ShouldShowObjective(title)
	if ( self.db.profile.showCurrentZone ) then
		local _, _, _, _, _, _, zone = Quixote:GetQuestByName(title)
		if ( zone == self.currentZone ) then
			return true
		end
	end
	if ( self.db.profile.hideOldObjectives > 0 ) then
		local minTime = GetTime() - ( self.db.profile.hideOldObjectives * 60 )
		if ( self.questTimes[title] ) and ( self.questTimes[title] > minTime ) then
			--It's new, return true
			return true
		end
		--When we have no data, we assume the quest is old
		return false
	end
	return true
end

function SmartVisibility:OnQuest_Gained(title)
	if ( self.db.profile.newOnGained ) then
		self.questTimes[title] = GetTime()
	end
end

function SmartVisibility:OnQuest_Lost(title)
	self.questTimes[title] = nil
end

function SmartVisibility:OnQuest_Leaderboard_Update(title, _, _, desc)
	self.questTimes[title] = GetTime()
end